# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/rare.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_RareSettings(object):
    def setupUi(self, RareSettings):
        RareSettings.setObjectName("RareSettings")
        RareSettings.resize(623, 428)
        RareSettings.setWindowTitle("RareSettings")
        self.rare_layout = QtWidgets.QHBoxLayout(RareSettings)
        self.rare_layout.setObjectName("rare_layout")
        self.left_layout = QtWidgets.QVBoxLayout()
        self.left_layout.setObjectName("left_layout")
        self.interface_group = QtWidgets.QGroupBox(RareSettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.interface_group.sizePolicy().hasHeightForWidth())
        self.interface_group.setSizePolicy(sizePolicy)
        self.interface_group.setObjectName("interface_group")
        self.interface_layout = QtWidgets.QFormLayout(self.interface_group)
        self.interface_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.interface_layout.setObjectName("interface_layout")
        self.lang_label = QtWidgets.QLabel(self.interface_group)
        self.lang_label.setObjectName("lang_label")
        self.interface_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.lang_label)
        self.lang_select = QtWidgets.QComboBox(self.interface_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lang_select.sizePolicy().hasHeightForWidth())
        self.lang_select.setSizePolicy(sizePolicy)
        self.lang_select.setObjectName("lang_select")
        self.interface_layout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lang_select)
        self.color_label = QtWidgets.QLabel(self.interface_group)
        self.color_label.setObjectName("color_label")
        self.interface_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.color_label)
        self.color_select = QtWidgets.QComboBox(self.interface_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.color_select.sizePolicy().hasHeightForWidth())
        self.color_select.setSizePolicy(sizePolicy)
        self.color_select.setObjectName("color_select")
        self.color_select.addItem("")
        self.interface_layout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.color_select)
        self.style_label = QtWidgets.QLabel(self.interface_group)
        self.style_label.setObjectName("style_label")
        self.interface_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.style_label)
        self.style_select = QtWidgets.QComboBox(self.interface_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.style_select.sizePolicy().hasHeightForWidth())
        self.style_select.setSizePolicy(sizePolicy)
        self.style_select.setObjectName("style_select")
        self.style_select.addItem("")
        self.interface_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.style_select)
        self.interface_info = QtWidgets.QLabel(self.interface_group)
        font = QtGui.QFont()
        font.setItalic(True)
        self.interface_info.setFont(font)
        self.interface_info.setWordWrap(True)
        self.interface_info.setObjectName("interface_info")
        self.interface_layout.setWidget(3, QtWidgets.QFormLayout.SpanningRole, self.interface_info)
        self.left_layout.addWidget(self.interface_group)
        self.settings_group = QtWidgets.QGroupBox(RareSettings)
        self.settings_group.setObjectName("settings_group")
        self.behavior_layout = QtWidgets.QGridLayout(self.settings_group)
        self.behavior_layout.setObjectName("behavior_layout")
        self.save_size = QtWidgets.QCheckBox(self.settings_group)
        self.save_size.setObjectName("save_size")
        self.behavior_layout.addWidget(self.save_size, 5, 0, 1, 1)
        self.notification = QtWidgets.QCheckBox(self.settings_group)
        self.notification.setObjectName("notification")
        self.behavior_layout.addWidget(self.notification, 4, 0, 1, 1)
        self.log_games = QtWidgets.QCheckBox(self.settings_group)
        self.log_games.setObjectName("log_games")
        self.behavior_layout.addWidget(self.log_games, 6, 0, 1, 1)
        self.sys_tray = QtWidgets.QCheckBox(self.settings_group)
        self.sys_tray.setObjectName("sys_tray")
        self.behavior_layout.addWidget(self.sys_tray, 0, 0, 1, 1)
        self.auto_update = QtWidgets.QCheckBox(self.settings_group)
        self.auto_update.setObjectName("auto_update")
        self.behavior_layout.addWidget(self.auto_update, 1, 0, 1, 1)
        self.confirm_start = QtWidgets.QCheckBox(self.settings_group)
        self.confirm_start.setObjectName("confirm_start")
        self.behavior_layout.addWidget(self.confirm_start, 2, 0, 1, 1)
        self.auto_sync_cloud = QtWidgets.QCheckBox(self.settings_group)
        self.auto_sync_cloud.setObjectName("auto_sync_cloud")
        self.behavior_layout.addWidget(self.auto_sync_cloud, 3, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.behavior_layout.addItem(spacerItem, 2, 1, 2, 1)
        self.left_layout.addWidget(self.settings_group)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.left_layout.addItem(spacerItem1)
        self.rare_layout.addLayout(self.left_layout)
        self.right_layout = QtWidgets.QVBoxLayout()
        self.right_layout.setObjectName("right_layout")
        self.log_dir_group = QtWidgets.QGroupBox(RareSettings)
        self.log_dir_group.setObjectName("log_dir_group")
        self.log_dir_layout = QtWidgets.QVBoxLayout(self.log_dir_group)
        self.log_dir_layout.setObjectName("log_dir_layout")
        self.log_dir_open_button = QtWidgets.QPushButton(self.log_dir_group)
        self.log_dir_open_button.setObjectName("log_dir_open_button")
        self.log_dir_layout.addWidget(self.log_dir_open_button)
        self.log_dir_clean_button = QtWidgets.QPushButton(self.log_dir_group)
        self.log_dir_clean_button.setObjectName("log_dir_clean_button")
        self.log_dir_layout.addWidget(self.log_dir_clean_button)
        self.log_dir_size_label = QtWidgets.QLabel(self.log_dir_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log_dir_size_label.sizePolicy().hasHeightForWidth())
        self.log_dir_size_label.setSizePolicy(sizePolicy)
        self.log_dir_size_label.setText("")
        self.log_dir_size_label.setWordWrap(True)
        self.log_dir_size_label.setObjectName("log_dir_size_label")
        self.log_dir_layout.addWidget(self.log_dir_size_label)
        self.right_layout.addWidget(self.log_dir_group)
        self.groupBox = QtWidgets.QGroupBox(RareSettings)
        self.groupBox.setObjectName("groupBox")
        self.shortcuts_layout = QtWidgets.QVBoxLayout(self.groupBox)
        self.shortcuts_layout.setObjectName("shortcuts_layout")
        self.desktop_link_btn = QtWidgets.QPushButton(self.groupBox)
        self.desktop_link_btn.setObjectName("desktop_link_btn")
        self.shortcuts_layout.addWidget(self.desktop_link_btn)
        self.startmenu_link_btn = QtWidgets.QPushButton(self.groupBox)
        self.startmenu_link_btn.setObjectName("startmenu_link_btn")
        self.shortcuts_layout.addWidget(self.startmenu_link_btn)
        self.right_layout.addWidget(self.groupBox)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.right_layout.addItem(spacerItem2)
        self.rare_layout.addLayout(self.right_layout)

        self.retranslateUi(RareSettings)

    def retranslateUi(self, RareSettings):
        _translate = QtCore.QCoreApplication.translate
        self.interface_group.setTitle(_translate("RareSettings", "Interface"))
        self.lang_label.setText(_translate("RareSettings", "Language"))
        self.color_label.setText(_translate("RareSettings", "Color scheme"))
        self.color_select.setItemText(0, _translate("RareSettings", "None"))
        self.style_label.setText(_translate("RareSettings", "Style sheet"))
        self.style_select.setItemText(0, _translate("RareSettings", "None"))
        self.interface_info.setText(_translate("RareSettings", "Restart Rare to apply changes."))
        self.settings_group.setTitle(_translate("RareSettings", "Behavior"))
        self.save_size.setText(_translate("RareSettings", "Restore window size on application startup"))
        self.notification.setText(_translate("RareSettings", "Show notifications when downloads complete"))
        self.log_games.setText(_translate("RareSettings", "Show console windows when launching games"))
        self.sys_tray.setText(_translate("RareSettings", "Exit to system tray"))
        self.auto_update.setText(_translate("RareSettings", "Queue game updates on application startup"))
        self.confirm_start.setText(_translate("RareSettings", "Confirm before launching games"))
        self.auto_sync_cloud.setText(_translate("RareSettings", "Automatically upload/download cloud saves"))
        self.log_dir_group.setTitle(_translate("RareSettings", "Logs"))
        self.log_dir_open_button.setText(_translate("RareSettings", "Open log folder"))
        self.log_dir_clean_button.setText(_translate("RareSettings", "Clean log folder"))
        self.groupBox.setTitle(_translate("RareSettings", "Shortcuts"))
        self.desktop_link_btn.setText(_translate("RareSettings", "Create on desktop"))
        self.startmenu_link_btn.setText(_translate("RareSettings", "Create in menu"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    RareSettings = QtWidgets.QWidget()
    ui = Ui_RareSettings()
    ui.setupUi(RareSettings)
    RareSettings.show()
    sys.exit(app.exec_())
