# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/integrations/import_group.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ImportGroup(object):
    def setupUi(self, ImportGroup):
        ImportGroup.setObjectName("ImportGroup")
        ImportGroup.resize(651, 218)
        ImportGroup.setWindowTitle("ImportGroup")
        ImportGroup.setWindowFilePath("")
        self.import_layout = QtWidgets.QFormLayout(ImportGroup)
        self.import_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.import_layout.setObjectName("import_layout")
        self.path_edit_label = QtWidgets.QLabel(ImportGroup)
        self.path_edit_label.setObjectName("path_edit_label")
        self.import_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.path_edit_label)
        self.app_name_label = QtWidgets.QLabel(ImportGroup)
        self.app_name_label.setObjectName("app_name_label")
        self.import_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.app_name_label)
        self.import_folder_label = QtWidgets.QLabel(ImportGroup)
        self.import_folder_label.setObjectName("import_folder_label")
        self.import_layout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.import_folder_label)
        self.import_folder_check = QtWidgets.QCheckBox(ImportGroup)
        font = QtGui.QFont()
        font.setItalic(True)
        self.import_folder_check.setFont(font)
        self.import_folder_check.setObjectName("import_folder_check")
        self.import_layout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.import_folder_check)
        self.import_dlcs_label = QtWidgets.QLabel(ImportGroup)
        self.import_dlcs_label.setObjectName("import_dlcs_label")
        self.import_layout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.import_dlcs_label)
        self.import_dlcs_check = QtWidgets.QCheckBox(ImportGroup)
        font = QtGui.QFont()
        font.setItalic(True)
        self.import_dlcs_check.setFont(font)
        self.import_dlcs_check.setObjectName("import_dlcs_check")
        self.import_layout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.import_dlcs_check)
        self.import_force_label = QtWidgets.QLabel(ImportGroup)
        self.import_force_label.setObjectName("import_force_label")
        self.import_layout.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.import_force_label)
        self.import_force_check = QtWidgets.QCheckBox(ImportGroup)
        font = QtGui.QFont()
        font.setItalic(True)
        self.import_force_check.setFont(font)
        self.import_force_check.setObjectName("import_force_check")
        self.import_layout.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.import_force_check)
        self.import_button_label = QtWidgets.QLabel(ImportGroup)
        self.import_button_label.setText("Error")
        self.import_button_label.setObjectName("import_button_label")
        self.import_layout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.import_button_label)
        self.button_info_layout = QtWidgets.QHBoxLayout()
        self.button_info_layout.setObjectName("button_info_layout")
        self.import_button = QtWidgets.QPushButton(ImportGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.import_button.sizePolicy().hasHeightForWidth())
        self.import_button.setSizePolicy(sizePolicy)
        self.import_button.setObjectName("import_button")
        self.button_info_layout.addWidget(self.import_button)
        self.import_layout.setLayout(6, QtWidgets.QFormLayout.FieldRole, self.button_info_layout)
        self.platform_label = QtWidgets.QLabel(ImportGroup)
        self.platform_label.setObjectName("platform_label")
        self.import_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.platform_label)
        self.platform_layout = QtWidgets.QHBoxLayout()
        self.platform_layout.setObjectName("platform_layout")
        self.platform_combo = QtWidgets.QComboBox(ImportGroup)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.platform_combo.sizePolicy().hasHeightForWidth())
        self.platform_combo.setSizePolicy(sizePolicy)
        self.platform_combo.setObjectName("platform_combo")
        self.platform_layout.addWidget(self.platform_combo)
        self.platform_tooltip = QtWidgets.QLabel(ImportGroup)
        font = QtGui.QFont()
        font.setItalic(True)
        self.platform_tooltip.setFont(font)
        self.platform_tooltip.setObjectName("platform_tooltip")
        self.platform_layout.addWidget(self.platform_tooltip)
        self.import_layout.setLayout(2, QtWidgets.QFormLayout.FieldRole, self.platform_layout)

        self.retranslateUi(ImportGroup)

    def retranslateUi(self, ImportGroup):
        _translate = QtCore.QCoreApplication.translate
        ImportGroup.setTitle(_translate("ImportGroup", "Import EGL game from a directory"))
        self.path_edit_label.setText(_translate("ImportGroup", "Installation path"))
        self.app_name_label.setText(_translate("ImportGroup", "Override app name"))
        self.import_folder_label.setText(_translate("ImportGroup", "Import all folders"))
        self.import_folder_check.setText(_translate("ImportGroup", "Scan the installation path for game folders and import them"))
        self.import_dlcs_label.setText(_translate("ImportGroup", "Import DLCs"))
        self.import_dlcs_check.setText(_translate("ImportGroup", "If a game has DLCs, try to import them too"))
        self.import_force_label.setText(_translate("ImportGroup", "Force import"))
        self.import_force_check.setText(_translate("ImportGroup", "Import game despite missing files"))
        self.import_button.setText(_translate("ImportGroup", "Import Game"))
        self.platform_label.setText(_translate("ImportGroup", "Platform"))
        self.platform_tooltip.setText(_translate("ImportGroup", "Select the native platform of the game"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ImportGroup = QtWidgets.QGroupBox()
    ui = Ui_ImportGroup()
    ui.setupUi(ImportGroup)
    ImportGroup.show()
    sys.exit(app.exec_())
