# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/downloads/download_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.8
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DownloadWidget(object):
    def setupUi(self, DownloadWidget):
        DownloadWidget.setObjectName("DownloadWidget")
        DownloadWidget.resize(490, 90)
        DownloadWidget.setWindowTitle("DownloadWidget")
        self.active_download_layout = QtWidgets.QGridLayout(DownloadWidget)
        self.active_download_layout.setContentsMargins(0, 0, 0, 0)
        self.active_download_layout.setObjectName("active_download_layout")
        self.downloaded = QtWidgets.QLabel(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.downloaded.sizePolicy().hasHeightForWidth())
        self.downloaded.setSizePolicy(sizePolicy)
        self.downloaded.setText("downloaded")
        self.downloaded.setObjectName("downloaded")
        self.active_download_layout.addWidget(self.downloaded, 2, 3, 1, 1)
        self.dl_speed_label = QtWidgets.QLabel(DownloadWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dl_speed_label.setFont(font)
        self.dl_speed_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dl_speed_label.setObjectName("dl_speed_label")
        self.active_download_layout.addWidget(self.dl_speed_label, 1, 2, 1, 1)
        self.dl_name_label = QtWidgets.QLabel(DownloadWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.dl_name_label.setFont(font)
        self.dl_name_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.dl_name_label.setObjectName("dl_name_label")
        self.active_download_layout.addWidget(self.dl_name_label, 0, 0, 1, 1)
        self.cache_used = QtWidgets.QLabel(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cache_used.sizePolicy().hasHeightForWidth())
        self.cache_used.setSizePolicy(sizePolicy)
        self.cache_used.setText("cache_used")
        self.cache_used.setObjectName("cache_used")
        self.active_download_layout.addWidget(self.cache_used, 2, 1, 1, 1)
        self.time_left = QtWidgets.QLabel(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.time_left.sizePolicy().hasHeightForWidth())
        self.time_left.setSizePolicy(sizePolicy)
        self.time_left.setText("time_left")
        self.time_left.setObjectName("time_left")
        self.active_download_layout.addWidget(self.time_left, 1, 1, 1, 1)
        self.downloaded_label = QtWidgets.QLabel(DownloadWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.downloaded_label.setFont(font)
        self.downloaded_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.downloaded_label.setObjectName("downloaded_label")
        self.active_download_layout.addWidget(self.downloaded_label, 2, 2, 1, 1)
        self.dl_speed = QtWidgets.QLabel(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dl_speed.sizePolicy().hasHeightForWidth())
        self.dl_speed.setSizePolicy(sizePolicy)
        self.dl_speed.setText("dl_speed")
        self.dl_speed.setObjectName("dl_speed")
        self.active_download_layout.addWidget(self.dl_speed, 1, 3, 1, 1)
        self.cache_used_label = QtWidgets.QLabel(DownloadWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.cache_used_label.setFont(font)
        self.cache_used_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.cache_used_label.setObjectName("cache_used_label")
        self.active_download_layout.addWidget(self.cache_used_label, 2, 0, 1, 1)
        self.dl_name = QtWidgets.QLabel(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.dl_name.sizePolicy().hasHeightForWidth())
        self.dl_name.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.dl_name.setFont(font)
        self.dl_name.setText("dl_name")
        self.dl_name.setObjectName("dl_name")
        self.active_download_layout.addWidget(self.dl_name, 0, 1, 1, 3)
        self.time_left_label = QtWidgets.QLabel(DownloadWidget)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.time_left_label.setFont(font)
        self.time_left_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.time_left_label.setObjectName("time_left_label")
        self.active_download_layout.addWidget(self.time_left_label, 1, 0, 1, 1)
        self.kill_button = QtWidgets.QPushButton(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.kill_button.sizePolicy().hasHeightForWidth())
        self.kill_button.setSizePolicy(sizePolicy)
        self.kill_button.setObjectName("kill_button")
        self.active_download_layout.addWidget(self.kill_button, 3, 4, 1, 1)
        self.progress_bar = QtWidgets.QProgressBar(DownloadWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.progress_bar.sizePolicy().hasHeightForWidth())
        self.progress_bar.setSizePolicy(sizePolicy)
        self.progress_bar.setProperty("value", 0)
        self.progress_bar.setObjectName("progress_bar")
        self.active_download_layout.addWidget(self.progress_bar, 3, 0, 1, 4)
        self.active_download_layout.setColumnStretch(1, 1)
        self.active_download_layout.setColumnStretch(3, 1)

        self.retranslateUi(DownloadWidget)

    def retranslateUi(self, DownloadWidget):
        _translate = QtCore.QCoreApplication.translate
        self.dl_speed_label.setText(_translate("DownloadWidget", "Download speed:"))
        self.dl_name_label.setText(_translate("DownloadWidget", "Current download:"))
        self.downloaded_label.setText(_translate("DownloadWidget", "Downloaded:"))
        self.cache_used_label.setText(_translate("DownloadWidget", "Cache size:"))
        self.time_left_label.setText(_translate("DownloadWidget", "Time left:"))
        self.kill_button.setText(_translate("DownloadWidget", "Stop Download"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DownloadWidget = QtWidgets.QWidget()
    ui = Ui_DownloadWidget()
    ui.setupUi(DownloadWidget)
    DownloadWidget.show()
    sys.exit(app.exec_())
