# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/dialogs/sync_save_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SyncSaveDialog(object):
    def setupUi(self, SyncSaveDialog):
        SyncSaveDialog.setObjectName("SyncSaveDialog")
        SyncSaveDialog.resize(648, 394)
        self.verticalLayout = QtWidgets.QVBoxLayout(SyncSaveDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.title_label = QtWidgets.QLabel(SyncSaveDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.title_label.sizePolicy().hasHeightForWidth())
        self.title_label.setSizePolicy(sizePolicy)
        self.title_label.setObjectName("title_label")
        self.verticalLayout.addWidget(self.title_label)
        self.sync_widget_layout = QtWidgets.QHBoxLayout()
        self.sync_widget_layout.setObjectName("sync_widget_layout")
        self.verticalLayout.addLayout(self.sync_widget_layout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.cancel_button = QtWidgets.QPushButton(SyncSaveDialog)
        self.cancel_button.setObjectName("cancel_button")
        self.horizontalLayout_2.addWidget(self.cancel_button)
        self.verticalLayout.addLayout(self.horizontalLayout_2)

        self.retranslateUi(SyncSaveDialog)

    def retranslateUi(self, SyncSaveDialog):
        _translate = QtCore.QCoreApplication.translate
        SyncSaveDialog.setWindowTitle(_translate("SyncSaveDialog", "Sync saves with cloud"))
        self.title_label.setText(_translate("SyncSaveDialog", "Select save, you want to use for "))
        self.cancel_button.setText(_translate("SyncSaveDialog", "Cancel"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SyncSaveDialog = QtWidgets.QDialog()
    ui = Ui_SyncSaveDialog()
    ui.setupUi(SyncSaveDialog)
    SyncSaveDialog.show()
    sys.exit(app.exec_())
