# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/games/import_sync/eos_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_EosWidget(object):
    def setupUi(self, EosWidget):
        EosWidget.setObjectName("EosWidget")
        EosWidget.resize(586, 146)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(EosWidget.sizePolicy().hasHeightForWidth())
        EosWidget.setSizePolicy(sizePolicy)
        EosWidget.setWindowTitle("GroupBox")
        self.eos_layout = QtWidgets.QHBoxLayout(EosWidget)
        self.eos_layout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.eos_layout.setObjectName("eos_layout")
        self.overlay_stack = QtWidgets.QStackedWidget(EosWidget)
        self.overlay_stack.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.overlay_stack.setFrameShadow(QtWidgets.QFrame.Raised)
        self.overlay_stack.setObjectName("overlay_stack")
        self.overlay_info_page = QtWidgets.QWidget()
        self.overlay_info_page.setObjectName("overlay_info_page")
        self.formLayout_3 = QtWidgets.QFormLayout(self.overlay_info_page)
        self.formLayout_3.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout_3.setFormAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.formLayout_3.setObjectName("formLayout_3")
        self.installed_version_info_lbl = QtWidgets.QLabel(self.overlay_info_page)
        self.installed_version_info_lbl.setObjectName("installed_version_info_lbl")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.installed_version_info_lbl)
        self.installed_version_lbl = QtWidgets.QLabel(self.overlay_info_page)
        self.installed_version_lbl.setText("error")
        self.installed_version_lbl.setObjectName("installed_version_lbl")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.installed_version_lbl)
        self.installed_path_info_lbl = QtWidgets.QLabel(self.overlay_info_page)
        self.installed_path_info_lbl.setObjectName("installed_path_info_lbl")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.installed_path_info_lbl)
        self.installed_path_lbl = QtWidgets.QLabel(self.overlay_info_page)
        self.installed_path_lbl.setText("error")
        self.installed_path_lbl.setObjectName("installed_path_lbl")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.installed_path_lbl)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uninstall_button = QtWidgets.QPushButton(self.overlay_info_page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uninstall_button.sizePolicy().hasHeightForWidth())
        self.uninstall_button.setSizePolicy(sizePolicy)
        self.uninstall_button.setMaximumSize(QtCore.QSize(150, 16777215))
        self.uninstall_button.setObjectName("uninstall_button")
        self.horizontalLayout.addWidget(self.uninstall_button)
        self.update_check_button = QtWidgets.QPushButton(self.overlay_info_page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.update_check_button.sizePolicy().hasHeightForWidth())
        self.update_check_button.setSizePolicy(sizePolicy)
        self.update_check_button.setMaximumSize(QtCore.QSize(150, 16777215))
        self.update_check_button.setObjectName("update_check_button")
        self.horizontalLayout.addWidget(self.update_check_button)
        self.update_button = QtWidgets.QPushButton(self.overlay_info_page)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.update_button.sizePolicy().hasHeightForWidth())
        self.update_button.setSizePolicy(sizePolicy)
        self.update_button.setMaximumSize(QtCore.QSize(150, 16777215))
        self.update_button.setObjectName("update_button")
        self.horizontalLayout.addWidget(self.update_button)
        self.formLayout_3.setLayout(3, QtWidgets.QFormLayout.SpanningRole, self.horizontalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(6, 6, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout_3.setItem(2, QtWidgets.QFormLayout.SpanningRole, spacerItem1)
        self.overlay_stack.addWidget(self.overlay_info_page)
        self.overlay_install_page = QtWidgets.QWidget()
        self.overlay_install_page.setObjectName("overlay_install_page")
        self.formLayout = QtWidgets.QFormLayout(self.overlay_install_page)
        self.formLayout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.formLayout.setFormAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.overlay_install_page)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.SpanningRole, self.label)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem2)
        self.install_button = QtWidgets.QPushButton(self.overlay_install_page)
        self.install_button.setObjectName("install_button")
        self.horizontalLayout_3.addWidget(self.install_button)
        self.formLayout.setLayout(2, QtWidgets.QFormLayout.SpanningRole, self.horizontalLayout_3)
        spacerItem3 = QtWidgets.QSpacerItem(6, 6, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.formLayout.setItem(1, QtWidgets.QFormLayout.SpanningRole, spacerItem3)
        self.overlay_stack.addWidget(self.overlay_install_page)
        self.eos_layout.addWidget(self.overlay_stack)
        self.enable_frame = QtWidgets.QFrame(EosWidget)
        self.enable_frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.enable_frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.enable_frame.setObjectName("enable_frame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.enable_frame)
        self.verticalLayout.setObjectName("verticalLayout")
        self.select_pfx_combo = QtWidgets.QComboBox(self.enable_frame)
        self.select_pfx_combo.setObjectName("select_pfx_combo")
        self.verticalLayout.addWidget(self.select_pfx_combo)
        self.enabled_cb = QtWidgets.QCheckBox(self.enable_frame)
        self.enabled_cb.setObjectName("enabled_cb")
        self.verticalLayout.addWidget(self.enabled_cb)
        self.enabled_info_label = QtWidgets.QLabel(self.enable_frame)
        font = QtGui.QFont()
        font.setItalic(True)
        self.enabled_info_label.setFont(font)
        self.enabled_info_label.setText("")
        self.enabled_info_label.setObjectName("enabled_info_label")
        self.verticalLayout.addWidget(self.enabled_info_label)
        self.eos_layout.addWidget(self.enable_frame)

        self.retranslateUi(EosWidget)
        self.overlay_stack.setCurrentIndex(0)

    def retranslateUi(self, EosWidget):
        _translate = QtCore.QCoreApplication.translate
        EosWidget.setTitle(_translate("EosWidget", "Epic Overlay"))
        self.installed_version_info_lbl.setText(_translate("EosWidget", "Version"))
        self.installed_path_info_lbl.setText(_translate("EosWidget", "Location"))
        self.uninstall_button.setText(_translate("EosWidget", "Uninstall"))
        self.update_check_button.setText(_translate("EosWidget", "Check for update"))
        self.update_button.setText(_translate("EosWidget", "Update"))
        self.label.setText(_translate("EosWidget", "Epic Overlay Services is not installed"))
        self.install_button.setText(_translate("EosWidget", "Install"))
        self.enabled_cb.setText(_translate("EosWidget", "Activated"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    EosWidget = QtWidgets.QGroupBox()
    ui = Ui_EosWidget()
    ui.setupUi(EosWidget)
    EosWidget.show()
    sys.exit(app.exec_())
