# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/downloads/queue_base_widget.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_QueueBaseWidget(object):
    def setupUi(self, QueueBaseWidget):
        QueueBaseWidget.setObjectName("QueueBaseWidget")
        QueueBaseWidget.resize(354, 72)
        QueueBaseWidget.setWindowTitle("QueueBaseWidget")
        QueueBaseWidget.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.queue_base_layout = QtWidgets.QHBoxLayout(QueueBaseWidget)
        self.queue_base_layout.setContentsMargins(0, 0, 0, 0)
        self.queue_base_layout.setObjectName("queue_base_layout")
        self.info_layout = QtWidgets.QHBoxLayout()
        self.info_layout.setObjectName("info_layout")
        self.queue_base_layout.addLayout(self.info_layout)
        self.move_buttons = QtWidgets.QWidget(QueueBaseWidget)
        self.move_buttons.setObjectName("move_buttons")
        self.move_buttons_layout = QtWidgets.QVBoxLayout(self.move_buttons)
        self.move_buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.move_buttons_layout.setObjectName("move_buttons_layout")
        self.move_up_button = QtWidgets.QPushButton(self.move_buttons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.move_up_button.sizePolicy().hasHeightForWidth())
        self.move_up_button.setSizePolicy(sizePolicy)
        self.move_up_button.setText("")
        self.move_up_button.setProperty("isSquare", 1)
        self.move_up_button.setObjectName("move_up_button")
        self.move_buttons_layout.addWidget(self.move_up_button, 0, QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.move_down_button = QtWidgets.QPushButton(self.move_buttons)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.move_down_button.sizePolicy().hasHeightForWidth())
        self.move_down_button.setSizePolicy(sizePolicy)
        self.move_down_button.setText("")
        self.move_down_button.setProperty("isSquare", 1)
        self.move_down_button.setObjectName("move_down_button")
        self.move_buttons_layout.addWidget(self.move_down_button, 0, QtCore.Qt.AlignLeft|QtCore.Qt.AlignBottom)
        self.queue_base_layout.addWidget(self.move_buttons)
        self.queue_buttons = QtWidgets.QWidget(QueueBaseWidget)
        self.queue_buttons.setObjectName("queue_buttons")
        self.queue_buttons_layout = QtWidgets.QVBoxLayout(self.queue_buttons)
        self.queue_buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.queue_buttons_layout.setObjectName("queue_buttons_layout")
        self.force_button = QtWidgets.QPushButton(self.queue_buttons)
        self.force_button.setObjectName("force_button")
        self.queue_buttons_layout.addWidget(self.force_button)
        self.remove_button = QtWidgets.QPushButton(self.queue_buttons)
        self.remove_button.setObjectName("remove_button")
        self.queue_buttons_layout.addWidget(self.remove_button, 0, QtCore.Qt.AlignTop)
        self.queue_base_layout.addWidget(self.queue_buttons)
        self.update_buttons = QtWidgets.QWidget(QueueBaseWidget)
        self.update_buttons.setObjectName("update_buttons")
        self.update_buttons_layout = QtWidgets.QVBoxLayout(self.update_buttons)
        self.update_buttons_layout.setContentsMargins(0, 0, 0, 0)
        self.update_buttons_layout.setObjectName("update_buttons_layout")
        self.update_button = QtWidgets.QPushButton(self.update_buttons)
        self.update_button.setObjectName("update_button")
        self.update_buttons_layout.addWidget(self.update_button, 0, QtCore.Qt.AlignTop)
        self.settings_button = QtWidgets.QPushButton(self.update_buttons)
        self.settings_button.setObjectName("settings_button")
        self.update_buttons_layout.addWidget(self.settings_button, 0, QtCore.Qt.AlignTop)
        self.queue_base_layout.addWidget(self.update_buttons)
        self.queue_base_layout.setStretch(0, 1)

        self.retranslateUi(QueueBaseWidget)

    def retranslateUi(self, QueueBaseWidget):
        _translate = QtCore.QCoreApplication.translate
        self.force_button.setText(_translate("QueueBaseWidget", "Download now"))
        self.remove_button.setText(_translate("QueueBaseWidget", "Remove from queue"))
        self.update_button.setText(_translate("QueueBaseWidget", "Update game"))
        self.settings_button.setText(_translate("QueueBaseWidget", "Update with settings"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    QueueBaseWidget = QtWidgets.QFrame()
    ui = Ui_QueueBaseWidget()
    ui.setupUi(QueueBaseWidget)
    QueueBaseWidget.show()
    sys.exit(app.exec_())
