# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/game_settings.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GameSettings(object):
    def setupUi(self, GameSettings):
        GameSettings.setObjectName("GameSettings")
        GameSettings.resize(505, 261)
        self.game_settings_layout = QtWidgets.QVBoxLayout(GameSettings)
        self.game_settings_layout.setObjectName("game_settings_layout")
        self.launch_settings_group = QtWidgets.QGroupBox(GameSettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.launch_settings_group.sizePolicy().hasHeightForWidth())
        self.launch_settings_group.setSizePolicy(sizePolicy)
        self.launch_settings_group.setObjectName("launch_settings_group")
        self.launch_settings_layout = QtWidgets.QFormLayout(self.launch_settings_group)
        self.launch_settings_layout.setLabelAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.launch_settings_layout.setObjectName("launch_settings_layout")
        self.skip_update_label = QtWidgets.QLabel(self.launch_settings_group)
        self.skip_update_label.setObjectName("skip_update_label")
        self.launch_settings_layout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.skip_update_label)
        self.skip_update = QtWidgets.QComboBox(self.launch_settings_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.skip_update.sizePolicy().hasHeightForWidth())
        self.skip_update.setSizePolicy(sizePolicy)
        self.skip_update.setObjectName("skip_update")
        self.skip_update.addItem("")
        self.skip_update.addItem("")
        self.skip_update.addItem("")
        self.launch_settings_layout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.skip_update)
        self.offline_label = QtWidgets.QLabel(self.launch_settings_group)
        self.offline_label.setObjectName("offline_label")
        self.launch_settings_layout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.offline_label)
        self.offline = QtWidgets.QComboBox(self.launch_settings_group)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.offline.sizePolicy().hasHeightForWidth())
        self.offline.setSizePolicy(sizePolicy)
        self.offline.setObjectName("offline")
        self.offline.addItem("")
        self.offline.addItem("")
        self.offline.addItem("")
        self.launch_settings_layout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.offline)
        self.launch_params_label = QtWidgets.QLabel(self.launch_settings_group)
        self.launch_params_label.setObjectName("launch_params_label")
        self.launch_settings_layout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.launch_params_label)
        self.launch_params = QtWidgets.QLineEdit(self.launch_settings_group)
        self.launch_params.setObjectName("launch_params")
        self.launch_settings_layout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.launch_params)
        self.game_settings_layout.addWidget(self.launch_settings_group)
        self.proton_layout = QtWidgets.QVBoxLayout()
        self.proton_layout.setObjectName("proton_layout")
        self.game_settings_layout.addLayout(self.proton_layout)
        self.linux_settings_widget = QtWidgets.QWidget(GameSettings)
        self.linux_settings_widget.setObjectName("linux_settings_widget")
        self.linux_settings_layout = QtWidgets.QVBoxLayout(self.linux_settings_widget)
        self.linux_settings_layout.setContentsMargins(0, 0, 0, 0)
        self.linux_settings_layout.setObjectName("linux_settings_layout")
        self.game_settings_layout.addWidget(self.linux_settings_widget)

        self.retranslateUi(GameSettings)

    def retranslateUi(self, GameSettings):
        _translate = QtCore.QCoreApplication.translate
        GameSettings.setWindowTitle(_translate("GameSettings", "GameSettings"))
        self.launch_settings_group.setTitle(_translate("GameSettings", "Launch Settings"))
        self.skip_update_label.setText(_translate("GameSettings", "Skip update check"))
        self.skip_update.setItemText(0, _translate("GameSettings", "Default"))
        self.skip_update.setItemText(1, _translate("GameSettings", "Yes"))
        self.skip_update.setItemText(2, _translate("GameSettings", "No"))
        self.offline_label.setText(_translate("GameSettings", "Offline mode"))
        self.offline.setItemText(0, _translate("GameSettings", "Default"))
        self.offline.setItemText(1, _translate("GameSettings", "Yes"))
        self.offline.setItemText(2, _translate("GameSettings", "No"))
        self.launch_params_label.setText(_translate("GameSettings", "Launch parameters"))
        self.launch_params.setPlaceholderText(_translate("GameSettings", "parameters"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    GameSettings = QtWidgets.QWidget()
    ui = Ui_GameSettings()
    ui.setupUi(GameSettings)
    GameSettings.show()
    sys.exit(app.exec_())
