# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/extra/console_env.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ConsoleEnv(object):
    def setupUi(self, ConsoleEnv):
        ConsoleEnv.setObjectName("ConsoleEnv")
        ConsoleEnv.resize(600, 400)
        self.layout = QtWidgets.QVBoxLayout(ConsoleEnv)
        self.layout.setObjectName("layout")
        self.table = QtWidgets.QTableWidget(ConsoleEnv)
        font = QtGui.QFont()
        font.setFamily("Monospace")
        self.table.setFont(font)
        self.table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.table.setAlternatingRowColors(True)
        self.table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.table.setCornerButtonEnabled(True)
        self.table.setColumnCount(2)
        self.table.setObjectName("table")
        self.table.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, item)
        self.table.horizontalHeader().setVisible(True)
        self.table.horizontalHeader().setStretchLastSection(True)
        self.table.verticalHeader().setVisible(False)
        self.layout.addWidget(self.table)
        self.buttons = QtWidgets.QDialogButtonBox(ConsoleEnv)
        self.buttons.setOrientation(QtCore.Qt.Horizontal)
        self.buttons.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttons.setObjectName("buttons")
        self.layout.addWidget(self.buttons)

        self.retranslateUi(ConsoleEnv)
        self.buttons.accepted.connect(ConsoleEnv.accept) # type: ignore
        self.buttons.rejected.connect(ConsoleEnv.reject) # type: ignore

    def retranslateUi(self, ConsoleEnv):
        _translate = QtCore.QCoreApplication.translate
        ConsoleEnv.setWindowTitle(_translate("ConsoleEnv", "Rare - Console Environment"))
        self.table.setSortingEnabled(True)
        item = self.table.horizontalHeaderItem(0)
        item.setText(_translate("ConsoleEnv", "Variable"))
        item = self.table.horizontalHeaderItem(1)
        item.setText(_translate("ConsoleEnv", "Value"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ConsoleEnv = QtWidgets.QDialog()
    ui = Ui_ConsoleEnv()
    ui.setupUi(ConsoleEnv)
    ConsoleEnv.show()
    sys.exit(app.exec_())
