# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/widgets/wrapper.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_WrapperSettings(object):
    def setupUi(self, WrapperSettings):
        WrapperSettings.setObjectName("WrapperSettings")
        WrapperSettings.resize(199, 30)
        WrapperSettings.setWindowTitle("WrapperSettings")
        self.wrapper_settings_layout = QtWidgets.QHBoxLayout(WrapperSettings)
        self.wrapper_settings_layout.setContentsMargins(0, 0, 0, 0)
        self.wrapper_settings_layout.setObjectName("wrapper_settings_layout")
        self.widget_stack = QtWidgets.QStackedWidget(WrapperSettings)
        self.widget_stack.setObjectName("widget_stack")
        self.label_page = QtWidgets.QWidget()
        self.label_page.setObjectName("label_page")
        self.label_page_layout = QtWidgets.QVBoxLayout(self.label_page)
        self.label_page_layout.setContentsMargins(0, 0, 0, 0)
        self.label_page_layout.setObjectName("label_page_layout")
        self.no_wrapper_label = QtWidgets.QLabel(self.label_page)
        self.no_wrapper_label.setObjectName("no_wrapper_label")
        self.label_page_layout.addWidget(self.no_wrapper_label)
        self.widget_stack.addWidget(self.label_page)
        self.wrapper_settings_layout.addWidget(self.widget_stack)
        self.add_button = QtWidgets.QPushButton(WrapperSettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.add_button.sizePolicy().hasHeightForWidth())
        self.add_button.setSizePolicy(sizePolicy)
        self.add_button.setObjectName("add_button")
        self.wrapper_settings_layout.addWidget(self.add_button, 0, QtCore.Qt.AlignTop)

        self.retranslateUi(WrapperSettings)
        self.widget_stack.setCurrentIndex(0)

    def retranslateUi(self, WrapperSettings):
        _translate = QtCore.QCoreApplication.translate
        self.no_wrapper_label.setText(_translate("WrapperSettings", "No wrapper added"))
        self.add_button.setText(_translate("WrapperSettings", "Add wrapper"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    WrapperSettings = QtWidgets.QWidget()
    ui = Ui_WrapperSettings()
    ui.setupUi(WrapperSettings)
    WrapperSettings.show()
    sys.exit(app.exec_())
