# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'rare/ui/components/tabs/settings/widgets/rpc.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_RPCSettings(object):
    def setupUi(self, RPCSettings):
        RPCSettings.setObjectName("RPCSettings")
        RPCSettings.resize(174, 146)
        RPCSettings.setWindowTitle("DiscordRPC")
        self.layout = QtWidgets.QGridLayout(RPCSettings)
        self.layout.setObjectName("layout")
        self.enable = QtWidgets.QComboBox(RPCSettings)
        self.enable.setObjectName("enable")
        self.enable.addItem("")
        self.enable.addItem("")
        self.enable.addItem("")
        self.layout.addWidget(self.enable, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(RPCSettings)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.layout.addWidget(self.label, 0, 0, 1, 1)
        self.show_game = QtWidgets.QCheckBox(RPCSettings)
        self.show_game.setObjectName("show_game")
        self.layout.addWidget(self.show_game, 1, 0, 1, 2)
        self.show_os = QtWidgets.QCheckBox(RPCSettings)
        self.show_os.setObjectName("show_os")
        self.layout.addWidget(self.show_os, 2, 0, 1, 2)
        self.show_time = QtWidgets.QCheckBox(RPCSettings)
        self.show_time.setObjectName("show_time")
        self.layout.addWidget(self.show_time, 3, 0, 1, 2)

        self.retranslateUi(RPCSettings)

    def retranslateUi(self, RPCSettings):
        _translate = QtCore.QCoreApplication.translate
        RPCSettings.setTitle(_translate("RPCSettings", "Discord RPC"))
        self.enable.setItemText(0, _translate("RPCSettings", "When Playing"))
        self.enable.setItemText(1, _translate("RPCSettings", "Always"))
        self.enable.setItemText(2, _translate("RPCSettings", "Never"))
        self.label.setText(_translate("RPCSettings", "Show"))
        self.show_game.setText(_translate("RPCSettings", "Show Game"))
        self.show_os.setText(_translate("RPCSettings", "Show OS"))
        self.show_time.setText(_translate("RPCSettings", "Show Time playing"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    RPCSettings = QtWidgets.QGroupBox()
    ui = Ui_RPCSettings()
    ui.setupUi(RPCSettings)
    RPCSettings.show()
    sys.exit(app.exec_())
